!=======================================================================
!                   _____________________________
!               \\//-----------------------------
!                )(   //=  //= //= //// //)) //
!               //\\ //_  //  //  /__/ //\\ //
!=======================================================================
!                   XEFFORT LIBRARY SAMPLES
!=======================================================================
! LVTest.f90 - demonstrates usage of ListView controls
!
! 2005. Xeffort.com
! Jugoslav Dujic (jdujic@uns.ns.ac.yu)
! 
! You are free to use the code for both commercial and non-commercial
! use. The code is provided as-is without warranties. You are free to
! distribute and modify this source code provided that the list of 
! original author(s) remains untouched and your revisions are indicated
! as such. Contributions are welcome.
!=======================================================================
MODULE LVTest

USE XEFFORT
USE DFWIN

IMPLICIT NONE

INCLUDE 'Resource.fd'

INTEGER, PARAMETER:: nI = 5000

TYPE MyType
   SEQUENCE
   CHARACTER(32) s
   INTEGER(4)    h
   REAL(8)       d(2)
   REAL(4)       f(2)
   INTEGER(4)    i
   INTEGER(2)    j
   INTEGER(1)    k
   INTEGER(1)    unused
END TYPE MyType

TYPE(MyType)::     xI(nI)

!========================================================================
CONTAINS
!========================================================================
!PURPOSE: Initialization function called by XFT library on app start.
LOGICAL FUNCTION XInit(szCmdLine,nCmdShow)
!DEC$ATTRIBUTES DEFAULT, DECORATE, ALIAS: 'XINIT':: XInit

IMPLICIT NONE

CHARACTER(*), INTENT(IN)::    szCmdLine
INTEGER, INTENT(IN)::         nCmdShow

LOGICAL::            bSt

IF (XCreateDialogApp(IDD_MAIN, idIcon=IDI_ICON_MAIN)) THEN

   !Set up the tab control
   bSt = XCtlSet(XW_FRAME, IDC_TAB1, 2, CTL_NUMITEMS)
   bSt = XCtlSet(XW_FRAME, IDC_TAB1, "Simple", 1)
   bSt = XCtlSet(XW_FRAME, IDC_TAB1, "Virtual", 2)
   bSt = XCtlSet(XW_FRAME, IDC_TAB1, IDC_LIST1, 1)
   bSt = XCtlSet(XW_FRAME, IDC_TAB1, IDC_LIST2, 2)

   CALL LVC_Initialize(IDC_LIST1)
   CALL LVC_Initialize(IDC_LIST2)

   bSt = XCtlSet(XW_FRAME, IDC_TAB1, 1, CTL_STATE)

!   bSt = XCtlCreate(XW_FRAME, IDC_LIST1, XCTL_LISTVIEW, CTS_OWNERDRAW, 10, 10, 525, 215, CTS_CLIENTEDGE)

   IF (XModalDialog(XW_FRAME).EQ.IDOK) THEN
      !Todo: place logic after dialog is exited via OK button here (XCtlGet)
   END IF
   Call XDestroyDialog(XW_FRAME)
END IF

!For a dialog-bassed application, the XInit may return .FALSE.
!Alternatively, use XModelessDialog, but return .TRUE. and set
!handlers for IDOK/IDCANCEL buttons.
XInit = .FALSE.

END FUNCTION XInit
!========================================================================
SUBROUTINE LVC_Initialize(idLV)

USE COMCTL32

INTEGER, INTENT(IN)::   idLV

CHARACTER(*), PARAMETER:: sFormat = "(a32,+a0,g10.3,d10.4, f10.0,e10.4, x4, j0, k3)"

LOGICAL::   bSt
TYPE(T_LVCOLUMN)::   LVC
TYPE(T_LVITEM)::     LVI
INTEGER::            i, hIl, hIcon, iSt, iStyle
CHARACTER(10)::      ss
INTEGER, PARAMETER:: idIcon(3) = (/IDI_BLUE, IDI_YELLOW, IDI_RED/)

!Set up the format and visibility
bSt = XCtlSet(XW_FRAME, idLV, sFormat, CTL_FORMAT)
bSt = XCtlSet(XW_FRAME, idLV, .TRUE., CTL_VISIBLE)

!Create the image list for the items
hIL = ImageList_Create(16, 12, ILC_MASK, 3, 0)
DO i=1,3
      hIcon = LoadImage(XGetInstance(), idIcon(i), IMAGE_ICON, 16, 12, LR_DEFAULTCOLOR)
      iSt = ImageList_ReplaceIcon(hIL, -1, hIcon)
END DO
bSt = XCtlSet(XW_FRAME, idLV, hIL, CTL_ICON)

!Text, background color, and font
bSt = XCtlSet(XW_FRAME, idLV, #800000, CTL_COLOR)
bSt = XCtlSet(XW_FRAME, idLV, #80FFFF, CTL_BKCOLOR)
bSt = XCtlSet(XW_FRAME, idLV, "t'Verdana'h10", CTL_FONT)

!Column captions
bSt = XCtlSet(XW_FRAME, idLV, "s", 1)
bSt = XCtlSet(XW_FRAME, idLV, "h", 2)
bSt = XCtlSet(XW_FRAME, idLV, "d(1)", 3)
bSt = XCtlSet(XW_FRAME, idLV, "d(2)", 4)
bSt = XCtlSet(XW_FRAME, idLV, "f(1)", 5)
bSt = XCtlSet(XW_FRAME, idLV, "f(2)", 6)
bSt = XCtlSet(XW_FRAME, idLV, "j", 7)
bSt = XCtlSet(XW_FRAME, idLV, "k", 8)

!Fill in the items depending on the type of control (simple/virtual)
iStyle = GetWindowLong(GetDlgItem(XW_FRAME%hWnd, idLV), GWL_STYLE)
IF (IAND(iStyle, LVS_OWNERDATA).EQ.0) THEN
      !Simple control -- add one item at a time
      DO i=1,nI
         xI(1) = MyType("Test "//XString(i), MOD(i,3)+1, (/ATAN(REAL(i)),REAL(i)/), &
            (/SQRT(REAL(i)), COS(REAL(i))/), i, 10000-i, i/2, 0)
         bSt = XCtlSet(XW_FRAME, idLV, XItem(xI(1)), i)
      END DO
ELSE
      !Virtual control -- set up the sizeof, numitems, and send the address of the first item
      bSt = XCtlSet(XW_FRAME, idLV, SIZEOF(xI(1)), CTL_SIZEOF)
      bSt = XCtlSet(XW_FRAME, idLV, nI, CTL_NUMITEMS)

      DO i=1,nI
         xI(i) = MyType("Test "//XString(i), MOD(i,3)+1, (/ATAN(REAL(i)),REAL(i)/), &
            (/SQRT(REAL(i)), COS(REAL(i))/), i, 10000-i, i/2, 0)
      END DO
      bSt = XCtlSet(XW_FRAME, idLV, XItem(xI), 0)
END IF

bSt = XCtlGet(XW_FRAME, idLV, ss, 2)

!Set up the callbacks
bSt = XCtlSetSub(XW_FRAME, idLV, List1_OnSelChange, CTL_SELCHANGE)
bSt = XCtlSetSub(XW_FRAME, idLV, List1_OnActivate, CTL_DBLCLICK)
bSt = XCtlSetSub(XW_FRAME, idLV, List1_OnColumnClick, CTL_COLUMNCLICK)

bSt = XCtlSet(XW_FRAME, idLV, XLV_AUTOHDR, CTL_RECTS)

END SUBROUTINE LVC_Initialize
!=======================================================================
!Callback for selection change (only does the )
SUBROUTINE List1_OnSelChange(xWnd, idCmd, nCode)

TYPE(X_WINDOW)::  xWnd
INTEGER, INTENT(in):: idCmd, nCode

INTEGER::            ist, nItems, jsel, iIndex
CHARACTER(6)::       ss
INTEGER, POINTER::   iSel(:)

INCLUDE "resource.fd"

iSt = XCtlGet(xWnd, idCmd, iSel, CTL_SELECTION)
iSt = XCtlGet(xWnd, idCmd, nItems, CTL_NUMITEMS)

iIndex = 0
jSel = 1
DO WHILE(XCtlGet(xWnd, idCmd, jSel, CTL_SELECTION, iIndex))
END DO

DEALLOCATE(iSel)

END SUBROUTINE List1_OnSelChange
!=======================================================================
!Double-click callback: change the text of selected items
SUBROUTINE List1_OnActivate(xWnd, idCmd, nCode)
 
TYPE(X_WINDOW)::  xWnd
INTEGER, INTENT(in):: idCmd, nCode

INTEGER::            ist, iStyle, i
CHARACTER(6)::       ss
INTEGER, POINTER::   iSel(:)
TYPE(MyType)::       Simple

INCLUDE "resource.fd"

iSt = XCtlGet(xWnd, idCmd, iSel, CTL_SELECTION)

iStyle = GetWindowLong(GetDlgItem(xWnd%hWnd, idCmd), GWL_STYLE)
IF (IAND(iStyle, LVS_OWNERDATA).EQ.0) THEN
   DO i=1,SIZE(iSel)
      iSt = XCtlGet(xWnd, idCmd, XItem(Simple), iSel(i))
      Simple%s = "Double-clicked"
      iSt = XCtlSet(xWnd, idCmd, XItem(Simple), iSel(i))
   END DO
ELSE
   DO i=1,SIZE(iSel)
      xI(iSel(i))%s = "Double-clicked"
   END DO
   iSt = XLVUpdate(xWnd, idCmd, iSel)
END IF
DEALLOCATE(iSel)

END SUBROUTINE List1_OnActivate
!=======================================================================
!Column click callback: disable the sorting of 2nd column
SUBROUTINE List1_OnColumnClick(xWnd, idCmd, iColumn)
 
TYPE(X_WINDOW)::  xWnd
INTEGER, INTENT(IN):: idCmd, iColumn

INCLUDE "resource.fd"

IF (iColumn.NE.2) THEN
      CALL XLV_OnColumnClick(xWnd, idCmd, iColumn)
END IF

END SUBROUTINE List1_OnColumnClick
!========================================================================
END MODULE LVTest

